/* globals __DEV__ */
import Phaser from 'phaser'
import Mushroom from '../sprites/Mushroom'
import config from './../config'

var socket =  '';
var isConnected = false;
var gameData = {};
var userID = 0;
var enemyID = 0;
var gameID = 0;
var playerLabel = '';
var peopleCount = 0;
var enemyPeopleCount = 0;
var money = 0;
var hasGetGame = false;
var loadingCircle = '';
var loadingTextElement = '';
var gameEnded = false;

export default class extends Phaser.State {
  init () {
  }
  preload () {
    this.stage.backgroundColor = '#4a98b9'
  }

  create () {
    this.routers = {
      smallRouter: {
        price: 50,
        radius: 45,
        asset: 'smallRouter',
        scaleFactor: 1,
      },
      bigRouter: {
        price: 100,
        radius: 65,
        asset: 'bigRouter',
        scaleFactor: 1,
      },
      start: {
        price: 0,
        radius: 25,
        asset: 'startRouter',
        scaleFactor: 1,
      }
    };
    this.objectsToTop = [];

    const loadingText = 'Waiting for player 2...'
    let loadingTextElement = this.add.text(this.game.world.centerX, 250, loadingText, {
      font: '50px Nunito',
      fill: config.labelColor,
      smoothed: false
    });
    loadingTextElement.x -= loadingTextElement.width / 2 ;

    let loadingCircle = this.add.image(this.game.world.centerX - 20, 400, 'loading_circle');
    loadingCircle.anchor.setTo(0.5,0.5);

    var loadingInterval = setInterval(() => {
      loadingCircle.angle += 3;
    }, 50);

    socket = new WebSocket('ws://pionlib.de:9000');
    var game = this.game;
    var that = this;

    socket.onopen = function(msg) {
        isConnected = true;

        socket.onclose = function(msg) {
            location.reload();
        };
        socket.onmessage = function(msg) {
            var obj = JSON.parse(msg.data);

            if (obj.type == 'createGame') {
                var timer = that.game.time.create(false);
                timer.loop(1000, that.updateStats, this);
                timer.start();

                userID = obj.userID;
                enemyID = obj.enemyID;
                gameID = obj.gameID;
                socket.send(JSON.stringify({
                    type: 'updateGame',
                    data: {
                        'gameID': gameID,
                        'userID': userID,
                        'name': game.nickname
                    }
                }));
            }

            var gameDataOld = gameData;

            if (obj.type == 'getGame') {
                gameData = obj.data;

                for (let entity in obj.data.users[userID].objects) {
                  if (typeof gameDataOld.users == "undefined" || typeof gameDataOld.users[userID].objects[obj.data.users[userID].objects[entity].id] == "undefined") {
                    that.printRouter(obj.data.users[userID].objects[entity].type, obj.data.users[userID].objects[entity].x, obj.data.users[userID].objects[entity].y, userID);
                  }
                }

                for (let entity in obj.data.users[enemyID].objects) {
                  if (typeof gameDataOld.users == "undefined" ||typeof gameDataOld.users[enemyID].objects[obj.data.users[enemyID].objects[entity].id] == "undefined") {
                    that.printRouter(obj.data.users[enemyID].objects[entity].type, obj.data.users[enemyID].objects[entity].x, obj.data.users[enemyID].objects[entity].y, enemyID);
                  }
                }

                if (!hasGetGame) {
                    hasGetGame = true;
                    loadingCircle.destroy();
                    loadingTextElement.destroy();
                    clearInterval(loadingInterval);
                    that.stage.backgroundColor = '#a1b96f'
                    that.drawUI(gameData);
                } else {
                    peopleCount.setText(Math.floor(gameData.users[userID].people) + ' (you)');
                    enemyPeopleCount.setText(Math.floor(gameData.users[enemyID].people) + ' (' + gameData.users[enemyID].name + ')');
                    money.setText('$ ' + Math.floor(gameData.users[userID].money));
                    var winValue = 1000;

                    if (!gameEnded && (gameData.users[userID].money > winValue || gameData.users[enemyID].money > winValue)) {
                        gameEnded = true;
                        if (gameData.users[enemyID].money > winValue) {
                            var msg = 'You lost against '+gameData.users[enemyID].name+'!';
                            var colorFinished = 'red';
                        } else {
                            var msg = 'You won against ' + gameData.users[enemyID].name+'!';
                            var colorFinished = 'green';
                        }

                        var emitter = that.game.add.emitter(that.game.world.centerX, 100, 200);
                        emitter.makeParticles(['wifi', 'smallRouter', 'startRouter', 'bigRouter']);
                        emitter.start(false, 5000, 20);
                        var choiseLabel = that.game.add.text(that.game.world.centerX, that.game.world.centerY - 80, msg,
                            { font: '45px Bungee', fill: colorFinished, smoothed: true});
                        choiseLabel.anchor.setTo(0.5, 0.5);
                    }
                }

                playerLabel.setText(that.game.nickname + ' vs ' + gameData.users[enemyID].name);
            }
        };
    }

    this.currentAction = 'none';
    this.newAction = 'none';
  }

  drawUI (gameData) {
    for (var mapObject in gameData.map) {
        this.game.add.sprite(gameData.map[mapObject].x, gameData.map[mapObject].y, gameData.map[mapObject].type);
    }

    for (var objectToTop in this.objectsToTop) {
      this.game.world.bringToTop(this.objectsToTop[objectToTop]);
    }

    const topBar = this.add.graphics(0, 0);
    topBar.beginFill(gameData.users[userID].color);
    topBar.drawRect(0, 0, 1000, 100);
    topBar.endFill();

    const gameLabel = this.add.text(30, 10, 'WiFi-Wars', {
      font: '40px Bungee',
      fill: config.labelColor,
      smoothed: true
    })

    playerLabel = this.add.text(30, 60, '', {
      font: '20px Nunito',
      fill: config.labelColor,
      smoothed: true
    })

    var peopleSprite = this.game.add.sprite(570, 15, 'people');
    peopleSprite.scale.setTo(0.35, 0.35);
    peopleCount = this.add.text(610, 20, '', {
      font: '20px Nunito',
      fill: config.labelColor,
      smoothed: false
    })

    var enemyPeopleSprite = this.game.add.sprite(570, 45, 'enemy_people');
    enemyPeopleSprite.scale.setTo(0.35, 0.35);
    enemyPeopleCount = this.add.text(610, 50, '', {
      font: '20px Nunito',
      fill: config.labelColor,
      smoothed: false
    })

    const moneyText = '$ 0'
    money = this.add.text(820, 20, moneyText, {
      font: '50px Nunito',
      fill: config.labelColor,
      smoothed: false
    })

    const bottomBar = this.add.graphics(0, 735);
    bottomBar.beginFill(gameData.users[userID].color);
    bottomBar.drawRect(0, 0, 1000, 65);
    bottomBar.endFill();

    this.bottomRibbons = [];
    this.drawRouterChooser('smallRouter');
    this.drawRouterChooser('bigRouter');
  }

  drawRouterChooser (type) {
    let price = this.routers[type].price;
    let asset = this.routers[type].asset + 'Buy';
    let setNewAction = function() {
      this.newAction = 'add' + type;
      this.addRouterType = type;
    }

    let xOffset = 130 + (this.bottomRibbons.length * 150);
    let yOffset = 60;
    let towerChooser = this.game.add.button(this.game.width - xOffset, this.game.height - yOffset, asset, setNewAction, this);
    let towerCostText = ' $ ' + price
    let towerCost = this.game.add.text(this.game.width - xOffset + towerChooser.width, this.game.height - yOffset + 10, towerCostText, {
      font: '25px Nunito',
      fill: config.labelColor,
      smoothed: false
    })
    towerCost.inputEnabled = true;
    towerCost.input.useHandCursor = true;
    towerCost.events.onInputDown.add(setNewAction, this);
    towerCost.events.onInputUp.add(setNewAction, this);
    this.bottomRibbons.push({
      type: type,
      price: price,
      button: towerChooser,
      label: towerCost,
    });
  }

  sendAddRouter (pointer, doubleTap) {
    socket.send(JSON.stringify({
        type: 'updateGame',
        data: {
            'gameID': gameID,
            'userID': userID,
            'addObject': {
              type: this.addRouterType,
              x: pointer.x,
              y: pointer.y,
            }
        }
    }));
    this.newAction = 'none';
    this.addRouterType = 'none';
  }

  printRouter (type, x, y, tmpUser) {
    let router = this.routers[type];
    let tower = this.add.graphics(x, y);
    tower.beginFill(gameData.users[tmpUser].color, 0.7);
    tower.drawCircle(0, 0, router.radius * 2);
    tower.endFill();
    let towerImg = game.cache.getImage(router.asset);
    let towerSprite = this.game.add.sprite(x - (towerImg.width*router.scaleFactor)/2, y - (towerImg.height*router.scaleFactor)/2, router.asset);
    towerSprite.scale.setTo(router.scaleFactor, router.scaleFactor);
    this.game.add.existing(tower)
    this.objectsToTop.push(tower)
    this.objectsToTop.push(towerSprite);
  }

  previewRouter (type) {
    let router = this.routers[type];
    let tower = this.add.graphics(this.game.input);
    tower.beginFill(gameData.users[userID].color, 0.4);
    tower.drawCircle(0, 0, router.radius * 2);
    tower.endFill();
    this.radiusPreview = this.game.add.existing(tower)

    let towerImg = game.cache.getImage(router.asset);
    this.cursorPreview = this.game.add.sprite(this.game.input.x - (towerImg.width*router.scaleFactor)/2, this.game.input.y - (towerImg.height*router.scaleFactor)/2, router.asset);
    this.cursorPreview.scale.setTo(router.scaleFactor, router.scaleFactor);
    this.cursorPreview.alpha = 0.6;
  }

  updatePreviewRouter (type) {
    let router = this.routers[type];
    let towerImg = game.cache.getImage(router.asset);

    if(this.radiusPreview != null) {
      this.radiusPreview.x = this.game.input.x;
      this.radiusPreview.y = this.game.input.y;
    }

    if(this.cursorPreview != null) {
      this.cursorPreview.x = this.game.input.x - (towerImg.width*router.scaleFactor)/2;
      this.cursorPreview.y = this.game.input.y - (towerImg.height*router.scaleFactor)/2;
    }
  }

  updateStats () {
      if (gameEnded) {
          return;
      }
    socket.send(JSON.stringify({
        type: 'getGame',
        data: {
            'gameID': gameID,
            'userID': userID,
            'calc': true,
        }
    }));
  }

  update () {
    if (this.currentAction != this.newAction) {
      this.game.input.onTap.removeAll();
      if(this.radiusPreview != null) {
        this.radiusPreview.destroy();
      }
      if(this.cursorPreview != null) {
        this.cursorPreview.destroy();
      }
      this.radiusPreview = null;
      this.cursorPreview = null;
    }

    if (this.newAction.substring(0, 3) == 'add') {
      if (this.currentAction != this.newAction) {
        this.game.input.onTap.add(this.sendAddRouter, this);
        this.previewRouter(this.addRouterType);
      }

      this.updatePreviewRouter(this.addRouterType);
    }

    this.currentAction = this.newAction;
  }

  render () {
    if (__DEV__) {
      // Debug example
      //this.game.debug.spriteInfo(this.mushroom, 32, 32)
    }
  }
}
