#!/usr/bin/env php
<?php

require_once('./websockets.php');

function dumpVar($_ = null)
{
    $args = func_get_args();
    foreach ($args as $argv) {
        if ($argv === true) {
            $r = 'true';
        } elseif($argv === false) {
            $r = 'false';
        } elseif ($argv === null) {
            $r = 'NULL';
        } else {
            $r = print_r($argv, true);
        }

        echo $r."\n";
    }
}

class wifiServer extends WebSocketServer {
    protected $typeRadios = [
        'smallRouter' => 45,
        'bigRouter' => 65,
        'start' => 25
    ];
    protected $maxBufferSize = 1048576;
    protected $objectsCount = 100;
    protected $userWithoutGame = [];
    protected $gameData = [];
    protected $coordinates = [];

    protected function createGame($gameID, $userA, $userB) {
        $objects = array();

        for ($i=0;$i<$this->objectsCount;$i++) {
            $type = 'house'.rand(0,2);

            if ($type === 'house0') {
                $people = 6;
            } elseif($type === 'house1') {
                $people = 12;
            } else {
                $people = 24;
            }

            $forTry = 0;
            for (;;) {
                $forTry++;

                if ($forTry == 5) {
                    continue 2;
                }

                $x = rand(0, 15);
                $y = rand(2, 10);

                if (!isset($this->coordinates[$gameID][$x][$y])) {
                    $objects[] = [
                        'id' => uniqid(),
                        'x' => $x * 60,
                        'y' => $y * 60,
                        'people' => $people,
                        'type' => $type,
                    ];

                    $forTry = 0;
                    $this->coordinates[$gameID][$x][$y] = true;
                    break;
                }

            }
        }

        $uniqidA = uniqid();
        $userA->color = '0x341da5';
        $userA->people = 0;
        $userA->money = 100;
        $userA->objects[$uniqidA] = [
            'id' => $uniqidA,
            'x' => 120,
            'y' => 180,
            'type' => 'start'
        ];

        $uniqidB = uniqid();
        $userB->color = '0x1d9b54';
        $userB->people = 0;
        $userB->money = 100;
        $userB->objects[$uniqidB] = [
            'id' => $uniqidB,
            'x' => 900,
            'y' => 600,
            'type' => 'start'
        ];

        $this->gameData[$gameID] = [
            'map' => $objects,
            'users' => [
                $userA->id => $userA,
                $userB->id => $userB
            ]
        ];
    }

    protected function process($user, $message) {
        $uniqid = uniqid();
        $object = json_decode($message, true);

        if ($object['type'] == 'getGame') {
            $gameID = $object['data']['gameID'];


            if (isset($object['data']['calc'])) {
                foreach($this->gameData[$gameID]['users'] as $updateUser) {
                    $peopleTotal = 0;

                    foreach ($this->gameData[$gameID]['map'] as $building) {
                        $buildingPoints = [
                            [$building['x'] + 30, $building['y'] + 15, false],
                            [$building['x'] + 15, $building['y'] + 30, false],
                            [$building['x'] + 45, $building['y'] + 30, false],
                            [$building['x'] + 30, $building['y'] + 45, false],
                        ];

                        foreach ($this->gameData[$gameID]['users'][$updateUser->id]->objects as $routerObject) {
                            foreach ($buildingPoints as $index => $buildingPoint) {
                                $aLength = abs($routerObject['x'] - $buildingPoint[0]);
                                $bLength = abs($routerObject['y'] - $buildingPoint[1]);
                                $distance = sqrt(pow($aLength, 2) + pow($bLength, 2));

                                if ($distance <= ($this->typeRadios[$routerObject['type']])) {
                                    $buildingPoints[$index][2] = true;
                                }
                            }

                        }

                        $c = 0;
                        foreach ($buildingPoints as $buildingPoint) {
                            if ($buildingPoint[2] === true) {
                                $c++;
                            }
                        }

                        $peopleTotal += $building['people'] / 4 * $c;
                    }

                    $this->gameData[$gameID]['users'][$updateUser->id]->people = $peopleTotal;
                    $oldMoney = $this->gameData[$gameID]['users'][$updateUser->id]->money;
                    $newMoney = $oldMoney + ($peopleTotal * 0.1);

                    $this->gameData[$gameID]['users'][$updateUser->id]->money = $newMoney;
                }
            }


            foreach($this->gameData[$gameID]['users'] as $updateUser) {
                $this->send($updateUser, json_encode(['type' => 'getGame', 'data' => $this->getSanitizedGameData($this->gameData[$gameID])]));
            }
        }

        if ($object['type'] == 'updateGame') {
            $gameID = $object['data']['gameID'];

            if (isset($object['data']['name'])) {
                $this->gameData[$gameID]['users'][$user->id]->name = $object['data']['name'];
            }

            if (isset($object['data']['addObject'])) {
                $objectX = $object['data']['addObject']['x'];
                $objectY = $object['data']['addObject']['y'];
                $setRouter = false;

                foreach ($this->gameData[$gameID]['users'][$user->id]->objects as $checkObject) {
                    $aLength = abs($objectX - $checkObject['x']);
                    $bLength = abs($objectY - $checkObject['y']);
                    $distance = sqrt(pow($aLength, 2) + pow($bLength, 2));

                    if ($distance < ($this->typeRadios[$object['data']['addObject']['type']] + $this->typeRadios[$checkObject['type']])) {
                        $setRouter = true;
                        break;
                    }
                }

                if ($setRouter) {
                    if ($object['data']['addObject']['type'] == 'smallRouter') {
                        $towerMoney = 50;
                    } elseif ($object['data']['addObject']['type'] == 'bigRouter') {
                        $towerMoney = 100;
                    } else {
                        $towerMoney = 0;
                    }

                    if ($this->gameData[$gameID]['users'][$user->id]->money >= $towerMoney) {
                        $this->gameData[$gameID]['users'][$user->id]->objects[$uniqid] = [
                            'id' => $uniqid,
                            'type' => $object['data']['addObject']['type'],
                            'x' => $objectX,
                            'y' => $objectY
                        ];

                        $this->gameData[$gameID]['users'][$user->id]->money = $this->gameData[$gameID]['users'][$user->id]->money - $towerMoney;
                    }

                }
            }

            foreach($this->gameData[$gameID]['users'] as $updateUser) {
                $this->send($updateUser, json_encode(['type' => 'getGame', 'data' => $this->getSanitizedGameData($this->gameData[$gameID])]));
            }
        }
    }

    protected function connected($user) {
        $userWithoutGame = reset($this->userWithoutGame);

        if (!empty($userWithoutGame)) {
            $gameID = uniqid();
            $this->createGame($gameID, $userWithoutGame, $user);
            $this->send($userWithoutGame, json_encode(['type' => 'createGame', 'gameID' => $gameID, 'userID' => $userWithoutGame->id, 'enemyID' => $user->id]));
            $this->send($user, json_encode(['type' => 'createGame', 'gameID' => $gameID, 'userID' => $user->id, 'enemyID' => $userWithoutGame->id]));
            $this->userWithoutGame = [];
        } else {
            $this->userWithoutGame[$user->id] = $user;
        }
    }

    protected function getSanitizedGameData($gameData) {
        $users = [];
        foreach ($gameData['users'] as $key => $user) {
            $users[$key]['color'] = $user->color;

            if (!empty($user->name)) {
                $users[$key]['name'] = $user->name;
            }

            $users[$key]['money'] = $user->money;
            $users[$key]['people'] = $user->people;

            if (!empty($user->objects)) {
                $users[$key]['objects'] = $user->objects;
            } else {
                $users[$key]['objects'] = [];
            }
        }

        return [
            'map' => $gameData['map'],
            'users' => $users,
        ];
    }
  
    protected function closed($user) {
        unset($this->userWithoutGame[$user->id]);
    }
}

$echo = new wifiServer("0.0.0.0", "9000");

try {
  $echo->run();
}
catch (Exception $e) {
  $echo->stdout($e->getMessage());
}
